#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.punk plugin.
"""

import pytest
from argparse import Namespace
from datetime import datetime, timedelta

from poppy.core.pipeline import Pipeline
from poppy.core.test import CommandTestCase, TaskTestCase

from roc.punk.tasks import SBMReport, SBMQuery
from roc.punk.tests.test_punk import PunkTest
from roc.dingo.models.data import SbmLog


def getSbmData():
    today = datetime.now()

    # Define all possible sbm objects
    nb_days = [5, 10, 10]
    sbm_types = [1, 1, 2]

    dates = [
        today
        - timedelta(
            days=n, hours=today.hour, minutes=today.minute, seconds=today.second
        )
        for n in nb_days
    ]

    sbms = [
        SbmLog(
            id=i,
            sbm_type=sbm_types[i],
            cuc_time=f"694407463:30{i}88",
            obt_time=dates[i],
            utc_time=dates[i],
            sbm_qf=2153,
            sbm_algo=1,
            status="Unknown",
        )
        for i, n in enumerate(nb_days)
    ]

    return sbms


def getFilteredSbmData(sbms, type, days):
    today = datetime.now()

    # Define the date to consider
    date_max = today - timedelta(
        days=days, hours=today.hour, minutes=today.minute, seconds=today.second
    )

    filtered_sbms = [
        sbm for sbm in sbms if sbm.utc_time >= date_max and sbm.sbm_type == type
    ]

    return filtered_sbms


def storeSbmData(sbms, session):
    for sbm in sbms:
        session.add(sbm)
    session.flush()


# REMEMBER TO TEST TASKS BEFORE COMMANDS


class TestSBMQuery(TaskTestCase):
    """
    With data generated by getSbmData(), report should notify
        `nb_results` events of `sbm_type`  for the latest `days`

    """

    @pytest.mark.parametrize(
        "sbm_type, days, nb_results",
        [
            (1, 8, 1),
            (1, 15, 2),
            (2, 8, 0),
            (2, 15, 1),
        ],
    )
    def test_sbm_query(self, sbm_type, days, nb_results):
        # --- set up arguments ---
        args = Namespace(type=sbm_type, days=days, dry_run=False)

        # --- database setup ---
        session = PunkTest.setup_session()

        # empty the table FileLog
        session.query(SbmLog).delete()
        session.flush()

        # --- ensure database is empty ---
        db_before = session.query(SbmLog).all()
        assert len(db_before) == 0

        # --- initialise db ---
        sbms = getSbmData()
        for sbm in sbms:
            session.add(sbm)
        session.flush()

        db_after = session.query(SbmLog).all()
        assert len(db_after) == len(sbms)

        # --- initialise task & pipeline --
        self.task = SBMQuery()
        self.task.pipeline = Pipeline(args)
        self.task.instantiate_targets()
        self.task.session = session

        # --- run the task ---
        self.run_task()

        # --- get results ---
        report_params = self.task.outputs["report_params"].data

        # --- make assertions ---
        if nb_results == 0:
            assert report_params is None
        else:
            assert "title" in report_params
            assert "labels" in report_params
            assert "assignee_id" in report_params
            assert "description" in report_params

            search_desc = f"{nb_results} SBM type {sbm_type} "
            assert search_desc in report_params["description"]

        session.close()


class TestSBMReport(TaskTestCase):
    def test_sbm_report(self):
        # --- set up arguments ---
        args = Namespace(token="xxx", dry_run=False)

        params = {}
        params["title"] = "Issue title"
        params["labels"] = "SBM"
        params["assignee_id"] = 142
        params["description"] = "Issue description"

        # --- initialise task & pipeline --
        self.task = SBMReport()
        self.task.pipeline = Pipeline(args)
        self.task.instantiate_targets()
        self.task.inputs["report_params"].data = params.copy()

        # --- run the task ---
        with pytest.raises(ValueError) as error:
            self.run_task()

        # A wrong token has been given
        assert "401" in str(error.value)


class TestSBMCmd(CommandTestCase):
    @pytest.mark.parametrize(
        "sbm_type, days, nb_results",
        [
            (1, 8, 1),
            (1, 15, 2),
            (2, 8, 0),
            (2, 15, 1),
        ],
    )
    def test_sbm_cmd(self, sbm_type, days, nb_results):
        """
        Dummy test

        :return:
        """

        sbms = getFilteredSbmData(getSbmData(), sbm_type, days)
        assert len(sbms) == nb_results

        # Db migration
        db_upgrade = ["pop", "-ll", "INFO", "db", "upgrade", "heads"]
        self.run_command(db_upgrade)

        # --- Search for SBM1 for the last 8 days ---
        main_command = [
            "pop",
            "-ll",
            "INFO",
            "punk",
            "sbm",
            "-t",
            f"{sbm_type}",
            "-d",
            f"{days}",
            "--token",
            "xxx",
        ]

        print(" ".join(main_command))

        # run the command
        self.run_command(main_command)

        # assert True if everything runs fine
        assert True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from poppy.pop.plugins import Plugin
from .punk import Punk

__all__ = ["descriptor_report"]

PunkTask = Plugin.manager["roc.punk"].task("descriptor_report")


@PunkTask.as_task
def descriptor_report(task):
    """
    Load all informations about pipeline, softwares and databases from the ROC
    database and retranscript it into an HTML format to be stored or converted
    into PDF reports.
    """
    # create the manager of reports
    punk = Punk()

    # load softwares
    punk.softwares_report(task.pipeline.output)


@PunkTask.as_task
def sbm1_report(task):
    """
    Send a notification with SBM1 detection of the week
    """
    # create the manager of reports
    punk = Punk()

    # generate report
    punk.sbm1_report()


# vim: set tw=79 :



# -*- coding: utf-8 -*-
from roc.punk.tasks.descriptor_report import *  # noqa: F401, F403
from roc.punk.tasks.sbm_report import *  # noqa: F401, F403
from roc.punk.tasks.sbm_query import *  # noqa: F401, F403



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import json
import os.path as osp

from jinja2 import Environment, FileSystemLoader

from poppy.core.tools.exceptions import DescriptorLoadError
from poppy.core.logger import logger
from poppy.core.conf import settings

__all__ = [
    "PLUGIN",
    "DESCRIPTOR",
    "PIPELINE_DATABASE",
]

# Name of the plugin
PLUGIN = "roc.punk"

# root directory of the module
_ROOT_DIRECTORY = osp.abspath(
    osp.join(
        osp.dirname(__file__),
    )
)


# Load pipeline database identifier
try:
    PIPELINE_DATABASE = settings.PIPELINE_DATABASE
except Exception:
    PIPELINE_DATABASE = "PIPELINE_DATABASE"
    logger.warning(
        f'settings.PIPELINE_DATABASE not defined for {__file__}, \
                     use "{PIPELINE_DATABASE}" by default!'
    )

try:
    TEST_DATABASE = settings.TEST_DATABASE
except Exception:
    TEST_DATABASE = "TEST_DATABASE"
    logger.warning(
        f'settings.TEST_DATABASE not defined for {__file__}, \
                     use "{TEST_DATABASE}" by default!'
    )

# Load descriptor file
descriptor_path = osp.join(_ROOT_DIRECTORY, "descriptor.json")
try:
    with open(descriptor_path, "r") as file_buffer:
        DESCRIPTOR = json.load(file_buffer)
except Exception:
    raise DescriptorLoadError(f"Loading {descriptor_path} has failed!")


# Setup jinja2 environment
JINJA_TEMPLATE_DIR = os.path.join(_ROOT_DIRECTORY, "templates")
JENV = Environment(loader=FileSystemLoader(JINJA_TEMPLATE_DIR))
# Jinja2 template for SOLO HK XML files
SOLO_HK_TEMPLATE = "solo_hk_xml.tpl"

# Access token
GITLAB_REPOSITORY = "https://gitlab.obspm.fr"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import argparse

from poppy.core.command import Command

from roc.punk.tasks.descriptor_report import DescriptorReport
from roc.punk.tasks.sbm_report import SBMReport
from roc.punk.tasks.sbm_query import SBMQuery

__all__ = []


def valid_data_path_type(arg):
    """Type function for argparse - an accessible path not empty"""

    # check is accessible
    ret = os.access(arg, os.R_OK)
    if not ret:
        raise argparse.ArgumentTypeError(
            "Argument must be a valid and readable data path"
        )

    # check if not empty
    listdir = os.listdir(arg)
    if len(listdir) == 0:
        raise argparse.ArgumentTypeError(
            "Argument data path must contain data. Directory is empty."
        )

    return arg


class PunkCommand(Command):
    """
    Command to manage the commands for the calibration software.
    """

    __command__ = "punk"
    __command_name__ = "punk"
    __parent__ = "master"
    __help__ = """Command relative to the PUNK module, responsible for making
        reports about the ROC pipeline, database and software."""


class PunkSoftwareReport(Command):
    """
    A command to run a calibration mode for a given software.
    """

    __command__ = "punk_sw_report"
    __command_name__ = "software"
    __parent__ = "punk"
    __parent_arguments__ = ["base"]
    __help__ = """Command for generating a report in HTML or PDF format about
        software and dataset in the ROC database"""

    def add_arguments(self, parser):
        parser.add_argument(
            "-o",
            "--output",
            help="""
                 The output directory
                 """,
            default="/tmp/",
            type=valid_data_path_type,
        )

    def setup_tasks(self, pipeline):
        # Set start task
        start = DescriptorReport()
        end = DescriptorReport()

        #  Build workflow of tasks
        pipeline | start

        # define the start/end task of the pipeline
        pipeline.start = start
        pipeline.end = end


class PunkSBMReport(Command):
    """
    A command to run a calibration mode for a given software.
    """

    __command__ = "punk_sbm_report"
    __command_name__ = "sbm"
    __parent__ = "punk"
    __parent_arguments__ = ["base"]
    __help__ = """Command for generating a weekly report
        with SBM1 detection of the week"""

    def add_arguments(self, parser):
        parser.add_argument(
            "--token",
            help="""
                 The Gitlab access token
                 """,
            type=str,
            required=True,
        )

        parser.add_argument(
            "-t",
            "--type",
            help="""
                 The SBM type
                 Possible values: 1, 2
                 """,
            type=int,
            default=1,
            choices=[1, 2],
        )

        parser.add_argument(
            "-d",
            "--days",
            help="""
                 The nb of days to take into account
                 """,
            type=int,
            default=8,
        )

        parser.add_argument(
            "-r",
            "--recipients",
            help="""
                 List of recipients of the SBM report. (Only gitlab account logins are accepted.)
                 """,
            nargs="+",
            type=str,
            default=["@xbonnin", "@dberard", "@maksimov", "@jsoucek", "@oalexandrova"],
        )

    def setup_tasks(self, pipeline):
        # Set start task
        query = SBMQuery()
        report = SBMReport()

        #  Build workflow of tasks
        pipeline | query | report

        # define the start/end task of the pipeline
        pipeline.start = query
        pipeline.end = report


# vim: set tw=79 :



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Functions used in all DINGO tests
"""

import os

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from roc.punk.constants import TEST_DATABASE


class PunkTest:
    def __init__(self):
        pass

    @staticmethod
    def load_configuration():
        from poppy.core.configuration import Configuration

        configuration = Configuration(os.getenv("PIPELINE_CONFIG_FILE", None))
        configuration.read()

        return configuration

    @staticmethod
    def setup_session():
        # Read config file
        conf = PunkTest.load_configuration()

        database_info = list(
            filter(
                lambda db: db["identifier"] == TEST_DATABASE, conf["pipeline.databases"]
            )
        )[0]

        # Create an Engine, which the Session will use for connection resources
        engine = create_engine(
            "{}://{}@{}/{}".format(
                database_info["login_info"]["vendor"],
                database_info["login_info"]["user"],
                database_info["login_info"]["address"],
                database_info["login_info"]["database"],
            )
        )
        # create a configured "Session" class
        Session = sessionmaker(bind=engine, autocommit=False)
        # create a Session
        session = Session()

        return session



#!/usr/bin/env python3
# -*- coding: utf-8 -*-



from poppy.core.task import Task


__all__ = ["DescriptorReport"]


class DescriptorReport(Task):
    """
    Class for managing reports from the ROC database.
    """

    plugin_name = "roc.punk"
    name = "descriptor_report"

    # def setup_inputs(self):
    #     """
    #     Init sessions, etc.
    #     """
    #
    #     # Get the output directory
    #     self.output = self.pipeline.get("output", args=True)
    #
    #     """
    #     Init the Jinja2 environment for loading templates with inheritance.
    #     """
    #
    #     # get the environment
    #     self.environment = Environment(loader=PackageLoader("roc.punk", "templates"))
    #
    # def get_software(self):
    #     """
    #     Get the list of software in the database for the specified version of
    #     the pipeline.
    #     """
    #     # get the pipeline
    #     # pipeline = self.roc.get_pipeline()
    #
    #     # query for software
    #     query = self.session.query(Plugin)
    #     # query = query.filter_by(pipeline=pipeline)
    #     return query.all()

    def run(self):
        """
        Make a report about software and their datasets.
        """

        raise DeprecationWarning("This task is deprecated and should be used.")

        # # Get roc connector
        # if not hasattr(self, "roc"):
        #     self.roc = Connector.manager[PIPELINE_DATABASE]
        #
        # # Get the database connection if needed
        # if not hasattr(self, "session"):
        #     self.session = self.roc.session
        #
        # # Initialize inputs
        # self.setup_inputs()
        #
        # # get software inside the ROC database for the specified version of
        # # the pipeline
        # software = self.get_software()
        #
        # # create a list of datasets from the software
        # datasets = set()
        # for current_software in software:
        #     for dataset in current_software.datasets:
        #         datasets.add(dataset)
        #
        # # get the template for the report about software
        # template = self.environment.get_template("software_report.html")
        #
        # # render the template
        # html = template.render(software_list=software, dataset_list=datasets)
        #
        # # get current datetime
        # now = datetime.datetime.now()
        #
        # # the filename prefix
        # filename = osp.join(
        #     self.output,
        #     "_".join(
        #         [
        #             self.roc.get_pipeline().release.release_version,
        #             now.strftime("%Y-%m-%d-%H-%M-%S"),
        #         ]
        #     ),
        # )
        #
        # # store into a file
        # with open(".".join([filename, "html"]), "w") as f:
        #     f.write(html)
        #
        # # write the PDF
        # HTML(string=html).write_pdf(".".join([filename, "pdf"]))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import requests
import urllib
from poppy.core.logger import logger
from poppy.core.task import Task
from poppy.core.target import BaseTarget

from roc.punk.constants import GITLAB_REPOSITORY


__all__ = ["SBMReport"]


class SBMReport(Task):
    """
    Class for reporting SBM events.
    """

    plugin_name = "roc.punk"
    name = "sbm_report"

    def add_targets(self):
        self.add_input(target_class=BaseTarget, identifier="report_params")

        self.add_output(target_class=BaseTarget, identifier="gitlab_response")

    def setup_inputs(self):
        """
        Init sessions, etc.
        """

        # Get the Gitlab access token
        self.token = self.pipeline.get("token", args=True)

        self.params = self.inputs["report_params"].data

        self.repository = GITLAB_REPOSITORY

        # Gitlab project in which create the issue
        self.project_id = 3336

    def run(self):
        """
        Create a gitlab issue.

        report_params is a dict to crate the query string described in
        https://docs.gitlab.com/ee/api/issues.html#new-issue
        """

        # Initialize inputs
        self.setup_inputs()

        if self.params is None:
            return None

        qs = urllib.parse.urlencode(self.params)
        url = f"{self.repository}/api/v4/projects/{self.project_id}/issues?"
        content = None

        try:
            response = requests.post(
                url, 
                headers={"PRIVATE-TOKEN": self.token},
                json=self.params
            )
            content = response.json()

            if response.status_code != 201:
                raise ValueError(
                    f"Wrong status response {response.status_code} : "
                    f"{content['message']}"
                )

            if type(content) is not dict:
                raise TypeError(f"Wrong format response : {content}")
            if "id" not in content:
                raise KeyError(f"No ID key in response : {content}")
            if type(content["id"]) is not int:
                raise TypeError(f"Wrong ID format : {content['id']}")

        except requests.exceptions.RequestException as e:
            logger.error(str(e))

        # Set outputs
        self.outputs["gitlab_response"].data = content

        return content



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime, timedelta

from sqlalchemy import and_

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task
from poppy.core.target import BaseTarget

from roc.punk.constants import PIPELINE_DATABASE
from roc.dingo.models.data import SbmLog

__all__ = ["SBMQuery"]


class SBMQuery(Task):
    """
    Class for querying SBM events.
    """

    plugin_name = "roc.punk"
    name = "sbm_query"

    def add_targets(self):
        self.add_output(target_class=BaseTarget, identifier="report_params")

    def setup_inputs(self):
        """
        Init sessions, etc.
        """

        # Get roc connector
        if not hasattr(self, "roc"):
            self.roc = Connector.manager[PIPELINE_DATABASE]

        # Get the database connection if needed
        if not hasattr(self, "session"):
            self.session = self.roc.session

        # Get the SBM type
        self.sbm_type = self.pipeline.get("type", args=True)

        # Get the number of days to take into account
        self.nb_days = self.pipeline.get("days", args=True)

        # Get list of recipients
        self.recipients = self.pipeline.get("recipients", args=True)

    def run(self):
        """
        Make a report about SBM event detected on-board for the past week.
        """

        # Initialize inputs
        self.setup_inputs()

        query = self.session.query(SbmLog)

        # Include only selected type
        type_filter = SbmLog.sbm_type == self.sbm_type  # noqa: E712

        # Only from the latest 8 days, starting at midnight
        now = datetime.now()
        last_week = now - timedelta(
            days=self.nb_days, hours=now.hour, minutes=now.minute, seconds=now.second
        )
        date_filter = SbmLog.obt_time >= last_week

        # Formatting period
        date_fmt = "%Y-%m-%d"
        period = f"{last_week.strftime(date_fmt)} to {now.strftime(date_fmt)}"

        # Query the database
        query = query.filter(and_(type_filter, date_filter))
        sbms = query.all()

        if len(sbms) == 0:
            self.outputs["report_params"].data = None
            logger.info(f"No SBM{self.sbm_type} for the period {period}")
            return

        logger.info(f"{len(sbms)} SBM{self.sbm_type} for the period {period}")

        sissi_url = "https://roc.pages.obspm.fr/MUSIC/sissi/events"
        sissi_url_dev = "https://roc-web.obspm.fr/sissi-dev/events"

        params = {
            "title": f"[SBM{self.sbm_type}] Report from {period}",
            "labels": "SBM",
            "assignee_id": 35,
            "description": (
                f"{len(sbms)} SBM{self.sbm_type} detections from {period}:\n\n"
            ),
        }
        params["description"] += " ".join(self.recipients) + "\n\n"
        for sbm in sbms:
            params["description"] += (
                f"- {sbm.utc_time} {sissi_url}/{sbm.id}\n"
            )

        # Set outputs
        self.outputs["report_params"].data = params
